@@ -202,7 +202,7 @@ namespace Beagle.Util {
 				XmlSerializer serializer = new XmlSerializer (type);
 				section = (Section) serializer.Deserialize (fs);
 			} catch (Exception e) {
-				Logger.Log.Error ("Could not load configuration from {0}: {1}", filename, e.Message);
+				Logger.Log.Error (e, "Could not load configuration from {0}:", filename);
 				if (fs != null)
 					fs.Close ();
 				if (current == null)
@@ -494,6 +494,12 @@ namespace Beagle.Util {
 				set { index_home_dir = value; }
 			}
 
+			private bool index_on_battery = true;
+			public bool IndexOnBattery {
+				get { return index_on_battery; }
+				set { index_on_battery = value; }
+			}
+
 			private ArrayList excludes = new ArrayList ();
 			[XmlArray]
 			[XmlArrayItem (ElementName="ExcludeItem", Type=typeof(ExcludeItem))]
@@ -525,6 +531,17 @@ namespace Beagle.Util {
 				return true;
 			}
 
+			[ConfigOption (Description="Toggles whether any data should be indexed if the system is on battery")]
+			internal bool IndexWhileOnBattery (out string output, string [] args)
+			{
+				if (index_on_battery)
+					output = "Data will not be indexed while on battery.";
+				else
+					output = "Data will be indexed while on battery.";
+				index_on_battery = !index_on_battery;
+				return true;
+			}
+
 			[ConfigOption (Description="Add a root path to be indexed", Params=1, ParamsDescription="A path")]
 			internal bool AddRoot (out string output, string [] args)
 			{
