@@ -37,7 +37,7 @@ using LNI = Lucene.Net.Index;
 
 namespace Beagle.Daemon.EvolutionMailDriver {
 
-	[QueryableFlavor (Name="Mail", Domain=QueryDomain.Local, RequireInotify=false)]
+	[QueryableFlavor (Name="EvolutionMail", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class EvolutionMailQueryable : LuceneQueryable {
 
 		public int polling_interval_in_seconds = 60;
@@ -55,9 +55,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		//    filters/indexes all attachments
 		// 3: Make email addresses non-keyword, add sanitized version
 		//    for searching for parts of an email address.
-		private const int INDEX_VERSION = 3;
+		// 4: Make the flags property mutable, and create a property
+		//    change Indexable when they change for IMAP generators.
+		private const int INDEX_VERSION = 4;
 
-		public EvolutionMailQueryable () : base ("MailIndex", INDEX_VERSION)
+		public EvolutionMailQueryable () : base ("EvolutionMailIndex", INDEX_VERSION)
 		{
 			this.local_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/local");
 			this.imap_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/imap");
@@ -177,23 +179,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 						       accountName, folderName, uid));
 		}
 
-		override protected double RelevancyMultiplier (Hit hit)
-		{
-			double t = 1.0;
-
-			// FIXME: We should probably be more careful
-			// about how we combine these two numbers into one,
-			// since the cardinal value of the score is used for
-			// comparisons with hits of other types.  It isn't
-			// sufficient to just worry about the ordinal relationship
-			// between two scores.
-			t *= HalfLifeMultiplierFromProperty (hit, 0.25,
-							     "fixme:received", "fixme:sentdate");
-
-
-			return t;
-		}
-
 		// An embarrassingly unscientific attempt at getting progress
 		// information from the mail backend as a whole.  Unfortunately
 		// the IMAP and mbox backends don't have a common unit of
