@@ -32,8 +32,9 @@ using System.Reflection;
 using System.Threading;
 using System.Text;
 using System.Text.RegularExpressions;
+using System.Runtime.InteropServices;
 
-using Gtk;
+using GLib;
 
 using Beagle;
 using Beagle.Util;
@@ -41,21 +42,22 @@ using Beagle.Daemon;
 
 class QueryTool {
 
-	static int count = 0;
-	static Query query = null;
-	static DateTime queryStartTime;
-	static DateTime lastQueryTime = DateTime.Now;
+	private static int count = 0;
+	private static Query query = null;
+	private static DateTime queryStartTime;
+	private static DateTime lastQueryTime = DateTime.Now;
 
+	private static MainLoop main_loop = null;
 	// CLI args
-	static bool keep_running = false;
-	static bool verbose = false;
-	static bool display_hits = true;
-	static bool flood = false;
-	static bool listener = false;
-	static DateTime start_date = DateTime.MinValue;
-	static DateTime end_date = DateTime.MinValue;
+	private static bool keep_running = false;
+	private static bool verbose = false;
+	private static bool display_hits = true;
+	private static bool flood = false;
+	private static bool listener = false;
+	private static DateTime start_date = DateTime.MinValue;
+	private static DateTime end_date = DateTime.MinValue;
 
-	static void OnHitsAdded (HitsAddedResponse response)
+	private static void OnHitsAdded (HitsAddedResponse response)
 	{
 		lastQueryTime = DateTime.Now;
 
@@ -97,7 +99,7 @@ class QueryTool {
 		}
 	}
 
-	static void OnHitsSubtracted (HitsSubtractedResponse response)
+	private static void OnHitsSubtracted (HitsSubtractedResponse response)
 	{
 		lastQueryTime = DateTime.Now;
 
@@ -112,7 +114,7 @@ class QueryTool {
 		}
 	}
 
-	static void OnFinished (FinishedResponse response)
+	private static void OnFinished (FinishedResponse response)
 	{
 		if (verbose) {
 			Console.WriteLine ("Elapsed time: {0:0.000}s",
@@ -123,7 +125,7 @@ class QueryTool {
 		if (flood)
 			SendQuery ();
 		else
-			Gtk.Application.Quit ();
+			main_loop.Quit ();
 	}
 
 	public static void PrintUsageAndExit () 
@@ -163,7 +165,7 @@ class QueryTool {
 		System.Environment.Exit (0);
 	}
 
-	static void ReadBackendMappings ()
+	private static void ReadBackendMappings ()
 	{
 		ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
 
@@ -203,16 +205,16 @@ class QueryTool {
 		}
 	}
 
-	static void OnClosed ()
+	private static void OnClosed ()
 	{
 		if (flood)
 			SendQuery ();
 		else
-			Gtk.Application.Quit ();
+			main_loop.Quit ();
 	}
 
-	static int query_counter = 0;
-	static void SendQuery ()
+	private static int query_counter = 0;
+	private static void SendQuery ()
 	{
 		++query_counter;
 		if (flood) {
@@ -231,9 +233,15 @@ class QueryTool {
 		}
 	}
 	
-	static void Main (string[] args) 
+	[DllImport("libgobject-2.0.so")]
+	static extern void g_type_init ();
+
+	public static void Main (string[] args) 
 	{
-		Gtk.Application.InitCheck ("beagle-query", ref args);
+		// Initialize GObject type system
+		g_type_init ();
+
+		main_loop = new MainLoop ();
 
 		if (args.Length == 0 || Array.IndexOf (args, "--help") > -1 || Array.IndexOf (args, "--usage") > -1)
 			PrintUsageAndExit ();
@@ -385,7 +393,7 @@ class QueryTool {
 
 		SendQuery ();
 
-		Gtk.Application.Run ();
+		main_loop.Run ();
 	}
 
 
